@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005 QNX Software Systems and others.
+ * Copyright (c) 2005, 2006 QNX Software Systems and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,19 +7,23 @@
  *
  * Contributors:
  *     QNX Software Systems - initial API and implementation
+ *     IBM Corporation
  *******************************************************************************/
 package org.eclipse.cdt.internal.ui.wizards.classwizard;
 
-import java.lang.reflect.InvocationTargetException;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.regex.Pattern;
 
 import org.eclipse.cdt.core.CCorePlugin;
 import org.eclipse.cdt.core.browser.AllTypesCache;
+import org.eclipse.cdt.core.browser.IQualifiedTypeName;
 import org.eclipse.cdt.core.browser.ITypeInfo;
 import org.eclipse.cdt.core.browser.ITypeReference;
-import org.eclipse.cdt.core.browser.ITypeSearchScope;
 import org.eclipse.cdt.core.browser.TypeSearchScope;
+import org.eclipse.cdt.core.dom.IPDOMManager;
+import org.eclipse.cdt.core.dom.ast.IBinding;
+import org.eclipse.cdt.core.dom.ast.gnu.cpp.GPPLanguage;
 import org.eclipse.cdt.core.model.CModelException;
 import org.eclipse.cdt.core.model.CoreModel;
 import org.eclipse.cdt.core.model.ICContainer;
@@ -28,8 +32,11 @@ import org.eclipse.cdt.core.model.ICProject;
 import org.eclipse.cdt.core.model.ISourceRoot;
 import org.eclipse.cdt.core.parser.IScannerInfo;
 import org.eclipse.cdt.core.parser.IScannerInfoProvider;
+import org.eclipse.cdt.internal.core.pdom.PDOM;
+import org.eclipse.cdt.internal.core.pdom.dom.PDOMBinding;
+import org.eclipse.cdt.internal.core.pdom.dom.PDOMNode;
+import org.eclipse.cdt.internal.core.pdom.dom.cpp.PDOMCPPLinkage;
 import org.eclipse.cdt.internal.ui.editor.CEditor;
-import org.eclipse.cdt.internal.ui.util.ExceptionHandler;
 import org.eclipse.cdt.internal.ui.viewsupport.IViewPartInputProvider;
 import org.eclipse.cdt.internal.ui.wizards.filewizard.NewSourceFileGenerator;
 import org.eclipse.cdt.ui.CUIPlugin;
@@ -38,12 +45,12 @@ import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
 import org.eclipse.core.resources.IWorkspaceRoot;
 import org.eclipse.core.resources.ResourcesPlugin;
+import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.core.runtime.IPath;
-import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.core.runtime.NullProgressMonitor;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.jface.operation.IRunnableContext;
-import org.eclipse.jface.operation.IRunnableWithProgress;
 import org.eclipse.jface.viewers.IStructuredSelection;
 import org.eclipse.ui.IEditorInput;
 import org.eclipse.ui.IEditorPart;
@@ -87,7 +94,7 @@ public class NewClassWizardUtil {
     public static ICContainer getSourceFolder(IPath path) {
         if (path == null)
             return null;
-        while (!path.isEmpty()) {
+        while (path.segmentCount() > 0) {
             IResource res = getWorkspaceRoot().findMember(path);
             if (res != null && res.exists()) {
                 int resType = res.getType();
@@ -271,35 +278,6 @@ public class NewClassWizardUtil {
         return ResourcesPlugin.getWorkspace().getRoot();
     }
     
-    /**
-     * Ensures the type cache is up to date.
-     * 
-     * @param context the runnable context
-     */
-    public static void prepareTypeCache(IRunnableContext context) {
-        final ITypeSearchScope scope = new TypeSearchScope(true);
-        if (!AllTypesCache.isCacheUpToDate(scope)) {
-            IRunnableWithProgress runnable = new IRunnableWithProgress() {
-                public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
-                    AllTypesCache.updateCache(scope, monitor);
-                    if (monitor.isCanceled()) {
-                        throw new InterruptedException();
-                    }
-                }
-            };
-            
-            try {
-                context.run(true, true, runnable);
-            } catch (InvocationTargetException e) {
-                String title = NewClassWizardMessages.getString("NewClassCreationWizardPage.getTypes.exception.title"); //$NON-NLS-1$
-                String message = NewClassWizardMessages.getString("NewClassCreationWizardPage.getTypes.exception.message"); //$NON-NLS-1$
-                ExceptionHandler.handle(e, title, message);
-            } catch (InterruptedException e) {
-                // cancelled by user
-            }
-        }
-    }
-
     /**
      * Resolve the location of the given class.
      * 
@@ -308,32 +286,6 @@ public class NewClassWizardUtil {
      * @return the class location, or <code>null</code> if not found
      */
     public static ITypeReference resolveClassLocation(ITypeInfo type, IRunnableContext context) {
-        prepareTypeCache(context);
-        
-        // resolve location of base class
-        if (type.getResolvedReference() == null) {
-            final ITypeInfo[] typesToResolve = new ITypeInfo[] { type };
-            IRunnableWithProgress runnable = new IRunnableWithProgress() {
-                public void run(IProgressMonitor progressMonitor) throws InvocationTargetException, InterruptedException {
-                    AllTypesCache.resolveTypeLocation(typesToResolve[0], progressMonitor);
-                    if (progressMonitor.isCanceled()) {
-                        throw new InterruptedException();
-                    }
-                }
-            };
-            
-            try {
-                context.run(true, true, runnable);
-            } catch (InvocationTargetException e) {
-                String title = NewClassWizardMessages.getString("NewClassCreationWizardPage.getTypes.exception.title"); //$NON-NLS-1$
-                String message = NewClassWizardMessages.getString("NewClassCreationWizardPage.getTypes.exception.message"); //$NON-NLS-1$
-                ExceptionHandler.handle(e, title, message);
-                return null;
-            } catch (InterruptedException e) {
-                // cancelled by user
-                return null;
-            }
-        }
         return type.getResolvedReference();
     }
 
@@ -346,10 +298,11 @@ public class NewClassWizardUtil {
      * @param project the given project
      * @return array of classes/structs
      */
-    public static ITypeInfo[] getReachableClasses(IProject project) {
+    public static ITypeInfo[] getReachableClasses(ICProject cProject) {
         ITypeInfo[] elements = AllTypesCache.getTypes(new TypeSearchScope(true), CLASS_TYPES);
         if (elements != null && elements.length > 0) {
-            if (project != null) {
+            if (cProject != null) {
+            	IProject project = cProject.getProject();
                 IScannerInfoProvider provider = CCorePlugin.getDefault().getScannerInfoProvider(project);
                 if (provider != null) {
                     //TODO get the scanner info for the actual source folder
@@ -359,7 +312,7 @@ public class NewClassWizardUtil {
                         List filteredTypes = new ArrayList();
                         for (int i = 0; i < elements.length; ++i) {
                             ITypeInfo baseType = elements[i];
-                            if (isTypeReachable(baseType, project, includePaths)) {
+                            if (isTypeReachable(baseType, cProject, includePaths)) {
                                 filteredTypes.add(baseType);
                             }
                         }
@@ -380,8 +333,8 @@ public class NewClassWizardUtil {
      * @param includePaths the include paths
      * @return <code>true</code> if the given type is found
      */
-    public static boolean isTypeReachable(ITypeInfo type, IProject project, String[] includePaths) {
-        IProject baseProject = type.getEnclosingProject();
+    public static boolean isTypeReachable(ITypeInfo type, ICProject project, String[] includePaths) {
+        ICProject baseProject = type.getEnclosingProject();
         if (baseProject != null) {
             if (baseProject.equals(project)) {
                 return true;
@@ -394,12 +347,109 @@ public class NewClassWizardUtil {
                         return true;
                 } else {
                     // we don't have the real location, so just check the project path
-                    if (baseProject.getLocation().isPrefixOf(includePath))
+                    if (baseProject.getProject().getLocation().isPrefixOf(includePath))
                         return true;
                 }
             }
         }
         return false;
     }
-    
+
+	public static final int SEARCH_MATCH_ERROR = -1;  // some error, search failed
+	public static final int SEARCH_MATCH_NOTFOUND = 0;  // no match found
+	public static final int SEARCH_MATCH_FOUND_EXACT = 1;   // exact match
+	public static final int SEARCH_MATCH_FOUND_EXACT_ANOTHER_TYPE = 2; // same name found, by different type
+	public static final int SEARCH_MATCH_FOUND_ANOTHER_NAMESPACE = 3; // same type name exits in different scope
+	public static final int SEARCH_MATCH_FOUND_ANOTHER_TYPE = 4; // same name used by another type in different scope
+
+	/**
+	 * Search for the given qualified name of the give
+	 * @param typeName  qualified name of the type to search
+	 * @param project  
+	 * @param pdomNodeType PDOMCPPLinkage type
+	 * @return
+	 */
+	public static int searchForCppType(IQualifiedTypeName typeName, ICProject project, int pdomNodeType) {
+		if(project == null) {
+			return SEARCH_MATCH_ERROR;
+		}
+		
+		String fullyQualifiedTypeName = typeName.getFullyQualifiedName();
+		
+		IPDOMManager pdomManager = CCorePlugin.getPDOMManager();
+		try {
+			PDOM pdom = (PDOM)pdomManager.getPDOM(project);	
+			IBinding[] bindings = pdom.findBindings(Pattern.compile(typeName.getName()), new NullProgressMonitor());
+			boolean sameTypeNameExists = false;
+			boolean sameNameDifferentTypeExists = false;
+			
+			for (int i = 0; i < bindings.length; ++i)
+			{
+				PDOMBinding binding = (PDOMBinding)bindings[i];
+				PDOMBinding pdomBinding = pdom.getLinkage(GPPLanguage.getDefault()).adaptBinding(binding);
+				
+				//get the fully qualified name of this binding
+				String bindingFullName = getBindingQualifiedName(pdomBinding);
+				
+				int currentNodeType = pdomBinding.getNodeType();
+				// full binding				
+				if (currentNodeType == pdomNodeType)
+				{						
+					if (bindingFullName.equals(fullyQualifiedTypeName))
+					{
+						return SEARCH_MATCH_FOUND_EXACT;
+					} else {
+						// same type , same name , but different name space
+						// see if there is an exact match;
+						sameTypeNameExists = true;
+					}
+				}
+				else if(currentNodeType == PDOMCPPLinkage.CPPCLASSTYPE || 
+						currentNodeType == PDOMCPPLinkage.CPPENUMERATION ||
+						currentNodeType == PDOMCPPLinkage.CPPNAMESPACE ||
+						currentNodeType == PDOMCPPLinkage.CPPTYPEDEF ||
+						currentNodeType == PDOMCPPLinkage.CPPBASICTYPE)
+				{						
+					if (bindingFullName.equals(fullyQualifiedTypeName))
+					{
+			        	return SEARCH_MATCH_FOUND_EXACT_ANOTHER_TYPE;
+			        } else {
+						// different type , same name , but different name space
+						sameNameDifferentTypeExists = true;
+			        }
+				}
+			}
+			if(sameTypeNameExists){
+				return SEARCH_MATCH_FOUND_ANOTHER_NAMESPACE;
+			}
+			
+			if(sameNameDifferentTypeExists){
+				return SEARCH_MATCH_FOUND_ANOTHER_TYPE;
+			}
+		} catch (CoreException e) {
+			return SEARCH_MATCH_ERROR;
+		}
+		return SEARCH_MATCH_NOTFOUND;
+	}
+
+	/**
+	 * Get the fully qualified name for a given PDOMBinding
+	 * @param pdomBinding
+	 * @return binding's fully qualified name
+	 * @throws CoreException
+	 */
+	public static String getBindingQualifiedName(PDOMBinding pdomBinding) throws CoreException
+	{
+		StringBuffer buf = new StringBuffer(pdomBinding.getName());	
+		PDOMNode parent = pdomBinding.getParentNode();
+		while (parent != null)
+		{
+			if (parent instanceof PDOMBinding)
+			{							
+				buf.insert(0, ((PDOMBinding)parent).getName() + "::");
+			}
+			parent = parent.getParentNode();
+		}
+		return buf.toString();
+	}    
 }
